@@ -1,17 +1,13 @@
 import numpy as np
 import pytest
 
-from sklearn.linear_model import LogisticRegression
 from sklearn.datasets import make_blobs
+from sklearn.linear_model import LogisticRegression
 
 from sklearn.utils.class_weight import compute_class_weight
 from sklearn.utils.class_weight import compute_sample_weight
-
-from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_almost_equal
-from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import assert_raise_message
-from sklearn.utils.testing import assert_equal
+from sklearn.utils._testing import assert_array_almost_equal
+from sklearn.utils._testing import assert_almost_equal
 
 
 def test_compute_class_weight():
@@ -30,17 +26,19 @@ def test_compute_class_weight_not_present():
     # Raise error when y does not contain all class labels
     classes = np.arange(4)
     y = np.asarray([0, 0, 0, 1, 1, 2])
-    assert_raises(ValueError, compute_class_weight, "balanced", classes, y)
+    with pytest.raises(ValueError):
+        compute_class_weight("balanced", classes, y)
     # Fix exception in error message formatting when missing label is a string
     # https://github.com/scikit-learn/scikit-learn/issues/8312
-    assert_raise_message(ValueError,
-                         'Class label label_not_present not present',
-                         compute_class_weight,
-                         {'label_not_present': 1.}, classes, y)
+    with pytest.raises(ValueError,
+                       match="Class label label_not_present not present"):
+        compute_class_weight({"label_not_present": 1.}, classes, y)
     # Raise error when y has items not in classes
     classes = np.arange(2)
-    assert_raises(ValueError, compute_class_weight, "balanced", classes, y)
-    assert_raises(ValueError, compute_class_weight, {0: 1., 1: 2.}, classes, y)
+    with pytest.raises(ValueError):
+        compute_class_weight("balanced", classes, y)
+    with pytest.raises(ValueError):
+        compute_class_weight({0: 1., 1: 2.}, classes, y)
 
 
 def test_compute_class_weight_dict():
@@ -57,16 +55,15 @@ def test_compute_class_weight_dict():
     # should get raised
     msg = 'Class label 4 not present.'
     class_weights = {0: 1.0, 1: 2.0, 2: 3.0, 4: 1.5}
-    assert_raise_message(ValueError, msg, compute_class_weight, class_weights,
-                         classes, y)
+    with pytest.raises(ValueError, match=msg):
+        compute_class_weight(class_weights, classes, y)
+
     msg = 'Class label -1 not present.'
     class_weights = {-1: 5.0, 0: 1.0, 1: 2.0, 2: 3.0}
-    assert_raise_message(ValueError, msg, compute_class_weight, class_weights,
-                         classes, y)
+    with pytest.raises(ValueError, match=msg):
+        compute_class_weight(class_weights, classes, y)
 
 
-@pytest.mark.filterwarnings('ignore: Default solver will be changed')  # 0.22
-@pytest.mark.filterwarnings('ignore: Default multi_class will')  # 0.22
 def test_compute_class_weight_invariance():
     # Test that results with class_weight="balanced" is invariant wrt
     # class imbalance if the number of samples is identical.
@@ -102,14 +99,14 @@ def test_compute_class_weight_balanced_negative():
     y = np.asarray([-1, -1, 0, 0, -2, -2])
 
     cw = compute_class_weight("balanced", classes, y)
-    assert_equal(len(cw), len(classes))
+    assert len(cw) == len(classes)
     assert_array_almost_equal(cw, np.array([1., 1., 1.]))
 
     # Test with unbalanced class labels.
     y = np.asarray([-1, 0, 0, -2, -2, -2])
 
     cw = compute_class_weight("balanced", classes, y)
-    assert_equal(len(cw), len(classes))
+    assert len(cw) == len(classes)
     class_counts = np.bincount(y + 2)
     assert_almost_equal(np.dot(cw, class_counts), y.shape[0])
     assert_array_almost_equal(cw, [2. / 3, 2., 1.])
@@ -135,16 +132,16 @@ def test_compute_class_weight_default():
 
     # Test for non specified weights
     cw = compute_class_weight(None, classes, y)
-    assert_equal(len(cw), classes_len)
+    assert len(cw) == classes_len
     assert_array_almost_equal(cw, np.ones(3))
 
     # Tests for partly specified weights
     cw = compute_class_weight({2: 1.5}, classes, y)
-    assert_equal(len(cw), classes_len)
+    assert len(cw) == classes_len
     assert_array_almost_equal(cw, [1.5, 1., 1.])
 
     cw = compute_class_weight({2: 1.5, 4: 0.5}, classes, y)
-    assert_equal(len(cw), classes_len)
+    assert len(cw) == classes_len
     assert_array_almost_equal(cw, [1.5, 1., 0.5])
 
 
@@ -195,39 +192,41 @@ def test_compute_sample_weight_with_subsample():
     # Test compute_sample_weight with subsamples specified.
     # Test with balanced classes and all samples present
     y = np.asarray([1, 1, 1, 2, 2, 2])
-    sample_weight = compute_sample_weight("balanced", y, range(6))
+    sample_weight = compute_sample_weight("balanced", y, indices=range(6))
     assert_array_almost_equal(sample_weight, [1., 1., 1., 1., 1., 1.])
 
     # Test with column vector of balanced classes and all samples present
     y = np.asarray([[1], [1], [1], [2], [2], [2]])
-    sample_weight = compute_sample_weight("balanced", y, range(6))
+    sample_weight = compute_sample_weight("balanced", y, indices=range(6))
     assert_array_almost_equal(sample_weight, [1., 1., 1., 1., 1., 1.])
 
     # Test with a subsample
     y = np.asarray([1, 1, 1, 2, 2, 2])
-    sample_weight = compute_sample_weight("balanced", y, range(4))
+    sample_weight = compute_sample_weight("balanced", y, indices=range(4))
     assert_array_almost_equal(sample_weight, [2. / 3, 2. / 3,
                                               2. / 3, 2., 2., 2.])
 
     # Test with a bootstrap subsample
     y = np.asarray([1, 1, 1, 2, 2, 2])
-    sample_weight = compute_sample_weight("balanced", y, [0, 1, 1, 2, 2, 3])
+    sample_weight = compute_sample_weight("balanced", y,
+                                          indices=[0, 1, 1, 2, 2, 3])
     expected_balanced = np.asarray([0.6, 0.6, 0.6, 3., 3., 3.])
     assert_array_almost_equal(sample_weight, expected_balanced)
 
     # Test with a bootstrap subsample for multi-output
     y = np.asarray([[1, 0], [1, 0], [1, 0], [2, 1], [2, 1], [2, 1]])
-    sample_weight = compute_sample_weight("balanced", y, [0, 1, 1, 2, 2, 3])
+    sample_weight = compute_sample_weight("balanced", y,
+                                          indices=[0, 1, 1, 2, 2, 3])
     assert_array_almost_equal(sample_weight, expected_balanced ** 2)
 
     # Test with a missing class
     y = np.asarray([1, 1, 1, 2, 2, 2, 3])
-    sample_weight = compute_sample_weight("balanced", y, range(6))
+    sample_weight = compute_sample_weight("balanced", y, indices=range(6))
     assert_array_almost_equal(sample_weight, [1., 1., 1., 1., 1., 1., 0.])
 
     # Test with a missing class for multi-output
     y = np.asarray([[1, 0], [1, 0], [1, 0], [2, 1], [2, 1], [2, 1], [2, 2]])
-    sample_weight = compute_sample_weight("balanced", y, range(6))
+    sample_weight = compute_sample_weight("balanced", y, indices=range(6))
     assert_array_almost_equal(sample_weight, [1., 1., 1., 1., 1., 1., 0.])
 
 
@@ -236,20 +235,27 @@ def test_compute_sample_weight_errors():
     # Invalid preset string
     y = np.asarray([1, 1, 1, 2, 2, 2])
     y_ = np.asarray([[1, 0], [1, 0], [1, 0], [2, 1], [2, 1], [2, 1]])
-    assert_raises(ValueError, compute_sample_weight, "ni", y)
-    assert_raises(ValueError, compute_sample_weight, "ni", y, range(4))
-    assert_raises(ValueError, compute_sample_weight, "ni", y_)
-    assert_raises(ValueError, compute_sample_weight, "ni", y_, range(4))
+
+    with pytest.raises(ValueError):
+        compute_sample_weight("ni", y)
+    with pytest.raises(ValueError):
+        compute_sample_weight("ni", y, indices=range(4))
+    with pytest.raises(ValueError):
+        compute_sample_weight("ni", y_)
+    with pytest.raises(ValueError):
+        compute_sample_weight("ni", y_, indices=range(4))
 
     # Not "balanced" for subsample
-    assert_raises(ValueError,
-                  compute_sample_weight, {1: 2, 2: 1}, y, range(4))
+    with pytest.raises(ValueError):
+        compute_sample_weight({1: 2, 2: 1}, y, indices=range(4))
 
     # Not a list or preset for multi-output
-    assert_raises(ValueError, compute_sample_weight, {1: 2, 2: 1}, y_)
+    with pytest.raises(ValueError):
+        compute_sample_weight({1: 2, 2: 1}, y_)
 
     # Incorrect length list for multi-output
-    assert_raises(ValueError, compute_sample_weight, [{1: 2, 2: 1}], y_)
+    with pytest.raises(ValueError):
+        compute_sample_weight([{1: 2, 2: 1}], y_)
 
 
 def test_compute_sample_weight_more_than_32():
